import java.util.ArrayList;

public class TextSearcherTest {
  
  static class Result {
    int cnt = 0;
    long start = 0;
    long end = 0;
    long time = 0;
    
    void add(Result r) {
      this.cnt += r.cnt;
      this.time += r.time;
    }
  }

  public static void main(String[] args) {
    // test1();
    test2();
  }
  
  private static void test1() {
    TextSearcher s = new BruteForceSearcher();
    Out.println(s.search("wettergott", "gott", false));
    Out.println(s.search("wettergott", "tt", false));
    Out.println(s.search("wettergott", "t", false));
    Out.println(s.search("er ist kaiser", "er", false));
    Out.println(s.search("er ist kaiser", "er", true));
  }
  
  private static void test2() {
    evaluate(
      readText("alice.txt").toLowerCase(), 
      // new String[] { "alice" } );
     readPatterns("patterns.txt"));
  }
  
  private static String readText(String filename) {
    In.open(filename);
    String text = In.readFile();
    In.close();
    return text;
  }
  
  private static String[] readPatterns(String filename) {
    In.open(filename);
    ArrayList<String> result = new ArrayList<String>();
    do {
      String line = In.readLine();
      if (In.done()) {
        result.add(line);
      }
    } while (In.done());
    In.close();
    return result.toArray(new String[0]);
  }
  
  private static void evaluate(String text, String[] patterns) {
    TextSearcher brute = new BruteForceSearcher();
    TextSearcher boyer = new BoyerMooreSearcher();
    Result total1 = new Result(), total2 = new Result();;
    Out.println(String.format("\n%-20s %4s %6s %4s %6s", 
      "Pattern", "#", "Brute", "#", "Boyer"));
    Out.println("---------------------------------------------");
    int cnt = 0;
    for (String p: patterns) {
      // if (++cnt == 9) break;
      Result r1 = evaluate(brute, text, p);
      Result r2 = evaluate(boyer, text, p);
      Out.println(String.format("%-20s %4d %6d %4d %6d", 
        p, r1.cnt, r1.time, r2.cnt, r2.time));
      total1.add(r1);
      total2.add(r2);
    }
    Out.println("---------------------------------------------");
    Out.println(String.format("%25d %6d %4d %6d",
      total1.cnt, total1.time, total2.cnt, total2.time));
  }
  
  private static Result evaluate(TextSearcher searcher, String text, String pattern) {
    final int REPEAT = 500;
    Result result = new Result();
    result.start = System.currentTimeMillis();
    for (int i=0; i < REPEAT; i++) {
      result.cnt = searcher.search(text, pattern, true);
    }
    result.end = System.currentTimeMillis();
    result.time = (result.end - result.start);
    return result;
  }
}
